@@ -15,12 +15,12 @@ tabs = ("General",
 
 class ModelProperties(QDialog):
 
-    def __init__(self, parent, model, main=None, onFinish=None):
+    def __init__(self, parent, deck, model, main=None, onFinish=None):
         QDialog.__init__(self, parent, Qt.Window)
         if not main:
             main = parent
         self.parent = main
-        self.deck = main.deck
+        self.deck = deck
         self.origModTime = self.deck.modified
         self.m = model
         self.onFinish = onFinish
@@ -33,7 +33,7 @@ class ModelProperties(QDialog):
         self.readData()
         self.show()
         self.undoName = _("Model Properties")
-        self.parent.deck.setUndoStart(self.undoName)
+        self.deck.setUndoStart(self.undoName)
         self.exec_()
 
     def readData(self):
@@ -289,10 +289,24 @@ class ModelProperties(QDialog):
         self.currentCard = self.m.cardModels[self.dialog.cardList.currentRow()]
         card = self.currentCard
         self.dialog.cardName.setText(card.name)
-        self.dialog.cardQuestion.setPlainText(card.qformat.replace("<br>", "\n"))
-        self.dialog.cardAnswer.setPlainText(card.aformat.replace("<br>", "\n"))
+        self.dialog.cardQuestion.setPlainText(card.qformat.replace("<br>", "<br>\n"))
+        self.dialog.cardAnswer.setPlainText(card.aformat.replace("<br>", "<br>\n"))
         self.dialog.questionInAnswer.setChecked(card.questionInAnswer)
         self.dialog.allowEmptyAnswer.setChecked(card.allowEmptyAnswer)
+        self.dialog.typeAnswer.clear()
+        self.fieldNames = self.deck.s.column0("""
+select fieldModels.name as n from fieldModels, cardModels
+where cardModels.modelId = fieldModels.modelId
+and cardModels.id = :id
+order by n""", id=card.id)
+        s = [_("Don't ask me to type in the answer")]
+        s += [_("Compare with field '%s'") % f for f in self.fieldNames]
+        self.dialog.typeAnswer.insertItems(0, QStringList(s))
+        try:
+            idx = self.fieldNames.index(card.typeAnswer)
+        except ValueError:
+            idx = -1
+        self.dialog.typeAnswer.setCurrentIndex(idx + 1)
         self.updateToggleButtonText(card)
 
     def enableCardMoveButtons(self):
@@ -321,14 +335,19 @@ class ModelProperties(QDialog):
             newname = _("Card %d") % (self.m.cardModels.index(card) + 1)
         self.updateField(card, 'name', newname)
         s = unicode(self.dialog.cardQuestion.toPlainText())
-        s = s.replace("\n", "<br>")
+        s = s.replace("<br>\n", "<br>")
         changed = self.updateField(card, 'qformat', s)
         s = unicode(self.dialog.cardAnswer.toPlainText())
-        s = s.replace("\n", "<br>")
+        s = s.replace("<br>\n", "<br>")
         changed2 = self.updateField(card, 'aformat', s)
         changed = changed or changed2
         self.updateField(card, 'questionInAnswer', self.dialog.questionInAnswer.isChecked())
         self.updateField(card, 'allowEmptyAnswer', self.dialog.allowEmptyAnswer.isChecked())
+        idx = self.dialog.typeAnswer.currentIndex()
+        if not idx:
+            self.updateField(card, 'typeAnswer', u"")
+        else:
+            self.updateField(card, 'typeAnswer', self.fieldNames[idx-1])
         if changed:
             # need to generate all question/answers for this card
             self.deck.updateCardsFromModel(self.m)
@@ -360,17 +379,17 @@ class ModelProperties(QDialog):
             return
         if len (self.m.cardModels) < 2:
             ui.utils.showWarning(
-                _("Please add a new card first."),
+                _("Please add a new template first."),
                 parent=self)
             return
         card = self.m.cardModels[row]
         count = self.deck.cardModelUseCount(card)
         if count:
             if not ui.utils.askUser(
-                _("This model is used by %d cards. If you delete it,\n"
+                _("This template is used by %d cards. If you delete it,\n"
                   "all the cards will be deleted too. If you just\n"
                   "want to prevent the creation of future cards with\n"
-                  "this model, please use the 'disable'  button\n"
+                  "this template, please use the 'disable'  button\n"
                   "instead.\n\nReally delete these cards?") % count,
                 parent=self):
                 return
@@ -388,7 +407,7 @@ class ModelProperties(QDialog):
                 active += 1
         if active < 2 and card.active:
             ui.utils.showWarning(
-                _("Please enable a different model first."),
+                _("Please enable a different template first."),
                 parent=self)
             return
         card.active = not card.active
@@ -445,6 +464,7 @@ class ModelProperties(QDialog):
     def reject(self):
         "Save user settings on close."
         # update properties
+        self.deck.startProgress()
         mname = unicode(self.dialog.name.text())
         if not mname:
             mname = _("Model")
@@ -472,10 +492,12 @@ class ModelProperties(QDialog):
             self.deck.setModified()
         # if changed, reset deck
         if self.origModTime != self.deck.modified:
-            self.parent.reset()
+            self.deck.updateTagsForModel(self.m)
+            ankiqt.mw.reset()
         if self.onFinish:
             self.onFinish()
-        self.parent.deck.setUndoEnd(self.undoName)
+        self.deck.setUndoEnd(self.undoName)
         # check again
-        self.parent.deck.haveJapanese = None
+        self.deck.haveJapanese = None
+        self.deck.finishProgress()
         QDialog.reject(self)
