@@ -36,8 +36,11 @@ class Sync(QThread):
         self.syncDeck()
 
     def error(self, error):
-        error = self.getErrorMessage(error)
-        self.emit(SIGNAL("showWarning"), error)
+        if error.data.get('type') == 'noResponse':
+            self.emit(SIGNAL("noSyncResponse"))
+        else:
+            error = self.getErrorMessage(error)
+            self.emit(SIGNAL("showWarning"), error)
         if self.onlyMerge:
             # new file needs cleaning up
             self.emit(SIGNAL("cleanNewDeck"))
@@ -49,10 +52,8 @@ class Sync(QThread):
             msg=_("Please double-check your username/password.")
         elif error.data.get('status') == "oldVersion":
             msg=_("The sync protocol has changed. Please upgrade.")
-        elif error.data.get('type') == "noResponse":
-            msg=_("Server is down or operation failed.")
         else:
-            msg=_("Unknown error: %s") % `error.data`
+            msg=_("Unknown error: %s" % `getattr(error, 'data')`)
         return msg
 
     def connect(self, *args):
@@ -81,7 +82,7 @@ class Sync(QThread):
                 self.setStatus("")
                 return
         timediff = abs(proxy.timestamp - time.time())
-        if timediff > 60:
+        if timediff > 300:
             self.emit(SIGNAL("syncClockOff"), timediff)
             return
         # reconnect
@@ -93,6 +94,7 @@ class Sync(QThread):
             # need to do anything?
             start = time.time()
             if client.prepareSync():
+                changes = True
                 # summary
                 self.setStatus(_("Fetching summary from server..."), 0)
                 sums = client.summaries()
@@ -115,11 +117,14 @@ class Sync(QThread):
                 self.deck.s.flush()
                 self.deck.s.commit()
             else:
+                changes = False
                 self.setStatus(_("No changes found."))
             # check sources
+            srcChanged = False
             if self.sourcesToCheck:
                 start = time.time()
                 self.setStatus(_("<br><br>Checking deck subscriptions..."))
+                srcChanged = False
                 for source in self.sourcesToCheck:
                     proxy.deckName = str(source)
                     msg = "%s:" % client.syncOneWayDeckName()
@@ -129,6 +134,7 @@ class Sync(QThread):
                     if not client.prepareOneWaySync():
                         self.setStatus(_(" * %s no changes found.") % msg)
                         continue
+                    srcChanged = True
                     self.setStatus(_(" * %s fetching payload...") % msg)
                     payload = proxy.genOneWayPayload(client.deck.lastSync)
                     self.setStatus(msg + _(" applied %d modified cards.") %
@@ -142,18 +148,20 @@ class Sync(QThread):
             # close and send signal to main thread
             self.deck.close()
             taken = time.time() - start
-            if taken < 2.5:
+            if (changes or srcChanged) and taken < 2.5:
                 time.sleep(2.5 - taken)
+            else:
+                time.sleep(0.25)
             self.emit(SIGNAL("syncFinished"))
         except Exception, e:
-            traceback.print_exc()
+            self.ok = False
+            #traceback.print_exc()
             self.deck.close()
             # cheap hack to ensure message is displayed
             err = `getattr(e, 'data', None) or e`
             self.setStatus(_("Syncing failed: %(a)s") % {
                 'a': err})
-            time.sleep(3)
-            self.emit(SIGNAL("syncFinished"))
+            self.error(e)
 
     def doBulkDownload(self, deckname):
         self.emit(SIGNAL("openSyncProgress"))
@@ -165,6 +173,7 @@ class Sync(QThread):
             client.sync()
         except:
             self.emit(SIGNAL("bulkSyncFailed"))
+        time.sleep(0.1)
         self.emit(SIGNAL("closeSyncProgress"))
 
     def bulkCallback(self, *args):
@@ -176,12 +185,16 @@ class Sync(QThread):
 class DeckChooser(QDialog):
 
     def __init__(self, parent, decks, create):
-        QDialog.__init__(self, parent)
+        QDialog.__init__(self, parent, Qt.Window)
         self.parent = parent
         self.decks = decks
         self.dialog = ankiqt.forms.syncdeck.Ui_DeckChooser()
         self.dialog.setupUi(self)
         self.create = create
+        if self.create:
+            self.dialog.topLabel.setText(_("<h1>Synchronize</h1>"))
+        else:
+            self.dialog.topLabel.setText(_("<h1>Open Online Deck</h1>"))
         if self.create:
             self.dialog.decks.addItem(QListWidgetItem(
                 _("Create '%s' on server") % self.parent.syncName))
@@ -191,7 +204,7 @@ class DeckChooser(QDialog):
             if self.create:
                 msg = _("Merge with '%s' on server") % name
             else:
-                msg = _("Copy '%s' from server") % name
+                msg = name
             item = QListWidgetItem(msg)
             self.dialog.decks.addItem(item)
         self.dialog.decks.setCurrentRow(0)
