@@ -8,7 +8,7 @@ from anki.sound import playFromText, stripSounds
 from anki.latex import renderLatex, stripLatex
 from ankiqt import ui
 
-import re, os, sys, urllib
+import re, os, sys, urllib, time
 import ankiqt
 
 def openLink(link):
@@ -55,18 +55,26 @@ def showText(text, parent=None):
               SIGNAL("clicked()"), d.accept)
     d.exec_()
 
-def askUser(text, parent=None):
+def askUser(text, parent=None, help=""):
     "Show a yes/no question. Return true if yes."
     if not parent:
         parent = ankiqt.mw
-    r = QMessageBox.question(parent, "Anki", text,
-                             QMessageBox.Yes | QMessageBox.No)
+    sb = QMessageBox.Yes | QMessageBox.No
+    if help:
+        sb |= QMessageBox.Help
+    while 1:
+        r = QMessageBox.question(parent, "Anki", text, sb,
+                                 QMessageBox.Yes)
+        if r == QMessageBox.Help:
+            openWikiLink(help)
+        else:
+            break
     return r == QMessageBox.Yes
 
 class GetTextDialog(QDialog):
 
     def __init__(self, parent, question, help=None, edit=None):
-        QDialog.__init__(self, parent)
+        QDialog.__init__(self, parent, Qt.Window)
         self.setWindowTitle("Anki")
         self.question = question
         self.help = help
@@ -167,21 +175,71 @@ def restoreSplitter(widget, key):
     if ankiqt.mw.config.get(key):
         widget.restoreState(ankiqt.mw.config[key])
 
+def saveHeader(widget, key):
+    key += "Header"
+    ankiqt.mw.config[key] = widget.saveState()
+
+def restoreHeader(widget, key):
+    key += "Header"
+    if ankiqt.mw.config.get(key):
+        widget.restoreState(ankiqt.mw.config[key])
+
 def mungeQA(deck, txt):
     txt = renderLatex(deck, txt)
     txt = stripSounds(txt)
-    def quote(match):
-        match = unicode(match.group(1))
-        if match.lower().startswith("http"):
-            src = match
-        else:
-            if sys.platform.startswith("win32"):
-                prefix = u"file:///"
-            else:
-                prefix = u"file://"
-            src = prefix + unicode(
-                urllib.quote(os.path.join(deck.mediaDir(
-                create=True), match).encode("utf-8")), "utf-8")
-        return 'img src="%s"' % src
-    txt = re.sub('img src="(.*?)"', quote, txt)
     return txt
+
+def getBase(deck):
+    if deck and deck.mediaDir():
+        if sys.platform.startswith("win32"):
+            prefix = u"file:///"
+        else:
+            prefix = u"file://"
+        base = prefix + unicode(
+            urllib.quote(deck.mediaDir().encode("utf-8")),
+            "utf-8")
+        return '<base href="%s/">' % base
+    else:
+        return ""
+
+class ProgressWin(object):
+
+    def __init__(self, parent, max=0, min=0, title=None):
+        if not title:
+            title = "Anki"
+        self.diag = QProgressDialog("", "", min, max, parent)
+        self.diag.setWindowTitle(title)
+        self.diag.setCancelButton(None)
+        self.diag.setAutoClose(False)
+        self.diag.setAutoReset(False)
+        self.diag.setMinimumDuration(0)
+        self.diag.show()
+        self.counter = min
+        self.min = min
+        self.max = max
+        self.lastTime = time.time()
+        self.app = QApplication.instance()
+        if max == 0:
+            self.diag.setLabelText(_("Processing..."))
+        self.app.processEvents()
+
+    def update(self, label=None, value=None):
+        self.app.processEvents()
+        #print self.min, self.counter, self.max, label, time.time() - self.lastTime
+        self.lastTime = time.time()
+        if label:
+            self.diag.setLabelText(label)
+        if value is None:
+            value = self.counter
+            self.counter += 1
+        else:
+            self.counter = value + 1
+        self.diag.setValue(value)
+        self.app.processEvents()
+
+    def finish(self):
+        if self.max:
+            self.diag.setValue(self.max)
+            self.app.processEvents()
+            time.sleep(0.1)
+        self.diag.cancel()
