@@ -24,9 +24,9 @@ setup(name='anki',
       author_email='anki@ichi2.net',
       url='http://ichi2.net/anki/index.html',
       license='GPLv3',
-      packages=find_packages(),
-      package_data={'anki':
-                    ['samples/*','locale/*/*/*']},
+      packages=["anki", "anki.features", "anki.features.chinese", "anki.importing"],
+      package_data={'anki': ['samples/*','locale/*/*/*'],
+		    'anki.features.chinese': ['unihan.db']},
       include_package_data=True,
       zip_safe=False,
       install_requires=[
