@@ -13,10 +13,6 @@ def test_stdmodels():
     deck = DeckStorage.Deck()
     deck.addModel(JapaneseModel())
     deck = DeckStorage.Deck()
-    deck.addModel(EnglishModel())
-    deck = DeckStorage.Deck()
-    deck.addModel(HeisigModel())
-    deck = DeckStorage.Deck()
     deck.addModel(CantoneseModel())
     deck = DeckStorage.Deck()
     deck.addModel(MandarinModel())
