@@ -26,14 +26,16 @@ class bdist_dmg(Command):
 		pass
 
 	def run(self):
-		self.run_command('py2app')
+                self.run_command('py2app')
+                if 'debug' in os.environ:
+                        return
                 # zlib
-		result = os.spawnvp(os.P_WAIT, 'hdiutil', (
+                result = os.spawnvp(os.P_WAIT, 'hdiutil', (
                         'hdiutil create -ov -format UDZO ' +
                         '-volname Anki -srcfolder dist ' +
                         '-o Anki.dmg -imagekey zlib-level=9').split())
-		if result is not 0:
-			raise Exception('dmg creation failed %d' % result)
+                if result is not 0:
+                        raise Exception('dmg creation failed %d' % result)
 
 APP = ['ankiqt/ankiqtmac.py']
 VERSION = appVersion
@@ -42,19 +44,23 @@ DATA_FILES = [
     'libanki/anki/locale',
     'ankiqt/ankiqt/locale',
     'kakasi',
-    'libanki/samples',
+    'audio',
     'ankiqt/imageformats',
     'libanki/anki/features/chinese/unihan.db',
     ]
 PLIST = dict(
-	CFBundleIdentifier = 'cx.repose.anki',
+	CFBundleIdentifier = 'net.ichi2.anki',
 	CFBundleName = 'Anki',
-	CFBundleDocumentTypes = [],
-	CFBundleLocalizations = ['en', 'ja', 'fr', 'de']
-)
+        CFBundleDocumentTypes=[dict(CFBundleTypeExtensions=["anki"],
+                                    CFBundleTypeName="Anki Deck",
+                                    CFBundleTypeRole="Editor",
+                                    CFBundleTypeIconFile="anki.icns")],
+	CFBundleLocalizations = ['en'],
+        )
 OPTIONS = {
 	'argv_emulation': True,
     'optimize': 0,
+        'alias': 'debug' in os.environ,
 	'plist': PLIST,
 	'iconfile': 'ankiqt/mac/anki.icns',
     "includes": ["sip", "cgi", "encodings", "encodings.utf_8",
@@ -64,6 +70,7 @@ OPTIONS = {
     'excludes': ['_gtkagg', '_tkagg', "_wxagg",
                  "wx", "_wx",
                  "Tkconstants", "Tkinter", "tcl", "pygame"],
+     #'frameworks': ['libmp3lame.dylib'],
     'dylib_excludes': ['libncurses.5.dylib',
                        '_wxagg.so',
                        '_tkagg.so',
